import time
from machine import Timer
from machine import Pin, ADC, I2C, SPI
import sys, gc
from st7735s import TFT
from sysfont import sysfont
import bme280

# Initialize SPI communication, for st7735s
time.sleep(1.0)	#デバイスの安定化を待つ
spi = SPI(0, baudrate=20000000, polarity=0, phase=0,
                        sck=Pin(2), mosi=Pin(3), miso=Pin(0))
tft=TFT(spi,11,10,1)	#インスタンス,DC,RES,CS

# TFT RGB ディスプレイを初期化
tft.initb2()
tft.rgb(False)
tft.fill(TFT.BLACK) #画面のクリア

# Initialize I2C communication, for BME280
i2c = I2C(id=0, scl=Pin(13), sda=Pin(12), freq=100000)
bme = bme280.BME280(i2c=i2c)

# 土壌湿気センサーを初期化
Pin(27,Pin.IN); moist = ADC(1)

# 水やりモーター、LED、Switch用GPIOの初期化
motor = Pin(16, Pin.OUT)
led = Pin(21, Pin.OUT)
sw = Pin(20, Pin.IN, Pin.PULL_UP)

global INTERVAL, WATERING_LENGTH, WATERING_THRESHOLD

# 各パラメータの読込み
with open('/parameter.dat', 'r') as f:	#ファイルに保存
    con=f.read()
    INTERVAL, WATERING_LENGTH, WATERING_THRESHOLD \
                               = map(int, con.split(','))

#タイマー割り込みの定義
state = 0
def toggle_mark(t):
    global state
    if state == 0:
        tft.fillcircle((110,10),8,TFT.CYAN); state = 1
    else:
        tft.fillcircle((110,10),8,TFT.BLACK); state = 0
    #print("割り込み")

t = Timer()
#t.init(period=1000, callback=toggle_mark)	#タイマー割込みの開始

def Get_Data():	# BME280および水分センサーからのデータを取得
    global temperature, humidity, pressure, moisture
    # Read sensor data
    t, p, h = bme.read_compensated_data()	#補償値
    temperature = t/100
    p = p // 256
    pi = p // 100
    pd = p - pi * 100
    pressure = float("{}.{:02d}".format(pi, pd))
    hi = h // 1024
    hd = h * 100 // 1024 - hi * 100
    humidity = float("{}.{:02d}".format(hi, hd))
    moisture = int(moist.read_u16())

for _ in range(5): Get_Data(); time.sleep(0.1)	#馴らしデータ採り

def Disp_Data():	# TFT RCB　ディスプレーに情報を表示する
    global temperature, humidity, pressure, moisture
    tft.fill(TFT.BLACK) # 画面のクリア
    tft.text((0, 18), f"T:{temperature:.1f}", TFT.YELLOW, sysfont, 3)
    tft.text((0,105), f"M:{moisture}", TFT.YELLOW, sysfont, 3)
    tft.text((0,47), f"H:{humidity:.1f}", TFT.YELLOW, sysfont, 3)
    tft.text((0,76), f"P:{pressure:.0f}", TFT.YELLOW, sysfont, 3)
    print(f"temp = {temperature}, humid = {humidity}, " +
                            f"press ={pressure}, moist = {moisture}")

def StateSW(slf):	# 10～300msで1、300ms～で2、2or5secで3、以外は0を返す
    st = time.ticks_ms()
    flag = 0; tm = 0
    tout = 2000 if slf == 0 else 5000
    while sw.value() == 0:
        tm = time.ticks_diff(time.ticks_ms(), st)
        if sw.value() == 1: break
        if tm >= tout: flag = 3; break	#Press 2or5秒
    if (tm >= 10) and (tm < 300): flag = 1; 	#短押し
    if (tm >= 300) and (tm < 1000): flag = 2; 	#ベタ押し
    return flag

def EditValue(item):	# Parameter値の編集
    global INTERVAL, WATERING_LENGTH, WATERING_THRESHOLD
    L = [["Interval"," of Action"," (minutes)","I",5,120,5], \
         ["WATERING_","LENGTH"," (seconds)","L",2,15,1], \
         ["WATERING_"," THRESHOLD","","M",15000,45000,100], \
         ["", "", "", "", 0, 0, 0]]
    Min = L[item][4]; Max = L[item][5]; Idv = L[item][6];
    tft.fill(TFT.BLACK) #画面のクリアと編集画面の表示
    tft.text((0, 0), "Set Up", TFT.YELLOW, sysfont, 3)
    tft.text((0, 30), L[item][0], TFT.YELLOW, sysfont, 2)
    tft.text((0, 50), L[item][1], TFT.YELLOW, sysfont, 2)
    tft.text((0, 75), L[item][2], TFT.YELLOW, sysfont, 2)
    val = vala = INTERVAL if item==0 else \
         (WATERING_LENGTH if item==1 else WATERING_THRESHOLD)
    tft.text((0,105), L[item][3]+f":{val}",TFT.YELLOW,sysfont,3)
    while sw.value() == 0: pass	#SW OFFを待つ
    
    while True:		#値の編集
        stat = StateSW(0)
        if stat == 3: tft.fill(TFT.BLACK); break	#2秒押しでNext
        if stat == 1: val = val + Idv		#短押しで加算
        elif stat == 2: val = val - Idv		#ベタ押しで減算
        #設定値は Min～Max の範囲とする
        val = Max if val > Max else (Min if val < Min else val)
        if val != vala:		#編集中の数字を表示
            tft.fillrect((0,105),(125,25),TFT.BLACK)
            tft.text((0,105),L[item][3]+f":{val}",TFT.YELLOW,\
                                                         sysfont,3)
            vala = val
        time.sleep(0.01)
    if item==0: INTERVAL = val	#Parameter値の代入
    elif item==1: WATERING_LENGTH = val
    elif item==2: WATERING_THRESHOLD = val
    
def SetParameter():	# 各種パラメーター値を編集する
    global INTERVAL, WATERING_LENGTH
    global WATERING_THRESHOLD, minita
    t.deinit()	#タイマー割込停止

    for i in range(3): EditValue(i)	#３つのパラメータの編集
    
    with open('/parameter.dat', 'w') as f:	#ファイルに保存
        f.write(str(INTERVAL) + ',' + \
            str(WATERING_LENGTH) + ',' + str(WATERING_THRESHOLD))
    print(INTERVAL, WATERING_LENGTH, WATERING_THRESHOLD)
    gc.collect()

def WaitTimDisplay():	# 再計測とActionまでの残り時間を表示する(分)
    global wait, minits, minita
    tft.fillrect((0,2),(98,10),TFT.BLACK); ts = wait // 60000
    tft.text((1,2),f"Until Next: {ts-minits}",TFT.YELLOW,sysfont,1)
    minita = minits		#1分に1回入ってくる
    gc.collect()


while True:
    try:
        Get_Data()	#センサー・データを取得
        Disp_Data()	#TFT LCD に情報を表示する

        #水やりが必要な状態ならモーターをWATERING_LENGTH秒間オンにする。
        if moisture > WATERING_THRESHOLD:
            motor.on(); led.on()
            time.sleep(WATERING_LENGTH)
            motor.off(); led.off()
        t.init(period=1000, callback=toggle_mark)	#タイマー割込再開
        while sw.value() == 0: pass	#SW OFFを待つ

        #================ 待機ループ ＆ SET UP待機 ================
        sttim = time.ticks_ms()	#現時間の記録
        waitmin = minita = INTERVAL	#待機時間指定(分)
        wait = waitmin * 60 * 1000	#msに換算
        # 全作業の待機時間(予定10～30分)
        while (tm := time.ticks_diff(time.ticks_ms(), sttim)) <= wait:
            #水やりのスレッショールド値を編集するかのチェック
            if StateSW(1) == 3:
                SetParameter(); break	#5秒押しで編集モード
            minits = tm // 60000;	#経過時間(分) 
            if minits != minita: WaitTimDisplay()	#次回CK残時間表示
            time.sleep(1.0)


    except KeyboardInterrupt:
        print("Stopping timer")
        t.deinit()  #タイマーを無効にして割り込みを停止
        sys.exit()
    
    except Exception as e:
        with open('/report.dat', 'a') as f: f.write(str(e) + "\n")
        machine.reset()
        #pass	#すべての例外を捕捉しプログラムの実行を継続する。
    
